package de.intevation.flys.artifacts.model.fixings;

import org.apache.log4j.Logger;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;
import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.model.CalculationResult;
import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.states.DefaultState.ComputeType;
import de.intevation.flys.utils.KMIndex;

/**
 * Facet to show the outliers in a fix calculation.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class FixOutlierFacet
extends      FixingsFacet
implements   FacetTypes {

    /** House logger. */
    private static Logger logger = Logger.getLogger(FixOutlierFacet.class);

    /** Trivial Constructor. */
    public FixOutlierFacet() {
    }


    /**
     * @param name
     */
    public FixOutlierFacet(String name, String description) {
        super(0, name, description, ComputeType.ADVANCE, null, null);
    }

    public FixOutlierFacet(int index, String name, String description) {
        super(index, name, description, ComputeType.ADVANCE, null, null);
    }


    /**
     * Returns the data this facet requires.
     *
     * @param artifact the owner artifact; needs to be a FLYSArtifact.
     * @param context  the CallContext; required to retrieve the value of
     * <i>currentKm</i>.
     *
     * @return an array of QW objects or null.
     */
    @Override
    public Object getData(Artifact artifact, CallContext context) {
        logger.debug("FixOutlierFacet.getData");

        if (artifact instanceof FLYSArtifact) {
            FLYSArtifact flys = (FLYSArtifact)artifact;

            CalculationResult res =
                (CalculationResult) flys.compute(context,
                                                 ComputeType.ADVANCE,
                                                 false);

            FixResult result = (FixResult) res.getData();
            double currentKm = getCurrentKm(context);

            KMIndex<QWI []>       kmQWs    = result.getOutliers();
            KMIndex.Entry<QWI []> qwsEntry = kmQWs.binarySearch(currentKm);

            QWI [] qws = null;
            if (qwsEntry != null) {
                qws = qwsEntry.getValue();

                if (logger.isDebugEnabled()) {
                    logger.debug("Found " + (qws != null ? qws.length : 0)
                        + " KMIndex.Entry for km " + currentKm);
                }
            }
            else {
                logger.debug("Found no KMIndex.Entry for km " + currentKm);
            }

            return qws;
        }

        logger.warn("Not an instance of FLYSArtifact.");
        return null;
    }


    /**
     * Create a deep copy of this Facet.
     * @return a deep copy.
     */
    @Override
    public FixOutlierFacet deepCopy() {
        FixOutlierFacet copy = new FixOutlierFacet();
        copy.set(this);
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
